/* 
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File interface.c - User interface implementation using libSCdialog 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "z80.h"
#include "mem.h"
#include "sam.h"
#include "util.h"
#include "version.h"
#include "dialog.h"
#include "fdi.h"
#include "interface.h"

int UiActive;

#define MAX_PATH	1024
char TheCwd[MAX_PATH];

/* UI Pointer status */

static int UIPointerDeltaX;
static int UIPointerDeltaY;
static int UIButtonState;

extern int disass;
extern int SAMPointerMode;

void
ui_init(void)
{
	/* User interface initialisation - called once at startup */

	DialogInit();

	UiActive = False;
}

void
ui_end(void)
{

	/* User interface Closedown */

	DialogCloseDown();

}

void
ui_enter(void)
{
	/* Enter the UI from the emulation */

	/* Get the Current Working Directory - useful for file i/o */


	if (getcwd(TheCwd, (size_t)MAX_PATH) == NULL)
		strcpy(TheCwd, "/");
	/* Halt the processor at the next frame interrupt */

	InterruptType = Z80_halt;
	UiActive = True;

	DialogStart();	

}

void
ui_leave(void)
{
	/* Leave the UI and return to the emulation */

	DialogEnd();

	/* Restart the processor */

	UiActive = False;
	InterruptType = Z80_restart;

}

void
ui_set_button_state(int ButtonState )
{
	/* Set Mouse Button status for UI */
	
	UIButtonState = ButtonState;

}

void
ui_set_pointer_position( int PointerDeltaX, int PointerDeltaY )
{

	/* Set Mouse Pointer position for UI */
	
	UIPointerDeltaX = PointerDeltaX;
	UIPointerDeltaY = PointerDeltaY;

}

void
ui_pointer_draw(void)
{
	PointerDraw();
}

void
ui_pointer_erase(void)
{
	PointerErase();
}

void
ui_event( void )
{
	/* Deal with UI events */


	if (SAMPointerMode) {
		PointerErase();

		PointerSetStatus(UIPointerDeltaX, UIPointerDeltaY, UIButtonState);
	}
	else {
		PointerSetStatus(-999, -999, -1);
	}

	if (DialogGetActiveDialogObject() == NULL) {
		int KeyEvent = GetSpecialKey();
		if (KeyEvent != KEY_NONE)
			ui_leave();
	}
	else {
		DialogCore();
	}

	if (SAMPointerMode) {
		PointerDraw();
	}
}



DialogObject *TestDialog, *SubTestDialog, *ListBoxDialog;

static char *AList[20] = {"Item01","Item02","Item03","Item04","Item05",
			  "Item06","Item07","Item08","Item09","Item10", 
			  "Item11","Item12","Item13","Item14","Item15", 
			  "Item16","Item17","Item18","Item19","Item20"}; 

static char TestInput[50] = "This is a test input box";

void
ui_test_CB(char *TheSelection)
{
	char TextBuffer[255];
	
	FillArea(0,180,255,GLYPH_HEIGHT,0x00, DM_OVER);
	sprintf(TextBuffer,"The Selection : %s",TheSelection);
	DrawString (0,180,TextBuffer,SHCOL_WHITE, DM_OVER);

}

void ui_file(char *TheFile)
{

	ui_test_end();

}


void ui_filesel(void)
{
	/* Test the file selector */
	FileSelectorOpen(TheCwd, "*.*", ui_file, NULL);

}


void
ui_test_start(void)
{

	ui_enter();
	TestDialog = FormCreate(50,50, 135, 50, NULL); 
	SubTestDialog = MenuCreate(10,10, TestDialog);
	ListBoxDialog = FormCreate(5,5,180,100, TestDialog);

	FormAddTextItem(TestDialog, 5, 5, "A Form with Buttons");
	FormAddButtonItem(TestDialog, 10, 20, "A Menu", FORM_DIALOG_BUTTON, SubTestDialog);
	FormAddButtonItem(TestDialog, 70, 20, "A List", FORM_FUNCTION_BUTTON, ui_filesel);

	FormAddTextItem(ListBoxDialog, 5, 5, "A Form with a List Box");
	FormAddListBoxItem(ListBoxDialog, 10, 20, 20, 10, 5, AList, ui_test_CB);
	FormAddListBoxItem(ListBoxDialog, 90, 20, 20, 10, 5, AList, ui_test_CB);
	FormAddButtonItem(ListBoxDialog, 10, 70, "A Button", FORM_FUNCTION_BUTTON, ui_filesel);
	FormAddInputBoxItem(ListBoxDialog, 90, 70, strlen(TestInput), 11, TestInput, NULL);
	MenuAddItem(SubTestDialog, MENU_DIALOG_ITEM, "Item 1", NULL);
	MenuAddItem(SubTestDialog, MENU_DIALOG_ITEM, "Item 2", NULL);
	MenuAddItem(SubTestDialog, MENU_DIALOG_ITEM, "Item 3", NULL);
	MenuAddItem(SubTestDialog, MENU_DIALOG_ITEM, "Item 4", NULL);
	MenuAddItem(SubTestDialog, MENU_FUNCTION_ITEM, "Quit", ui_test_end);


	ClearDisplay();
	DialogSetActiveDialogObject(TestDialog);

}

void
ui_test_end(void)
{
	DialogSetActiveDialogObject(NULL);
	MenuFree(SubTestDialog);
	FormFree(TestDialog);
	FormFree(ListBoxDialog);
	ui_leave();
}



DialogObject *
ui_make_about(void (*CBAbout)(void), DialogObject *AboutParent)
{
	/* Create an 'About' Dialog used both for the splash banner */
	/* and the main user interface 				    */

	DialogObject *TheAboutDialog;
	char buffer[255];
	
	sprintf(buffer, "SimCoupe vn %d.%d.%d",SIMCOUPE_Major,SIMCOUPE_Minor,SIMCOUPE_Patch);

	TheAboutDialog = FormCreate(50, 50, 156, 92, AboutParent);
	FormAddTextItem(TheAboutDialog, 28, 10, buffer);
	FormAddTextItem(TheAboutDialog, 28, 12, "_________________");
	FormAddTextItem(TheAboutDialog, 10, 30, "(c) 1997 A.J.Skillman");
		
	FormAddButtonItem(TheAboutDialog, 70,60, "OK", FORM_FUNCTION_BUTTON, CBAbout);
	
	return(TheAboutDialog);
}


static DialogObject *BannerDialog;

void
ui_banner(void)
{
	/* Use the UI to put up a welcoming banner */

	ui_enter();
	ClearDisplay();


	BannerDialog = ui_make_about(ui_end_banner, NULL);
	DialogSetActiveDialogObject(BannerDialog);
}

void 
ui_end_banner(void)
{
	DialogSetActiveDialogObject(NULL);
	FormFree(BannerDialog);
	ClearDisplay();
	ui_leave();
}

static DialogObject *TopMenuForm;
static DialogObject *FileMenu;
static DialogObject *SamToolsMenu;
static DialogObject *OptionsMenu;
static DialogObject *HelpMenu;
static DialogObject *SelectDriveMenu;
static DialogObject *AboutDialog;
static DialogObject *CreateImageDialog;


void
ui_start_debug(void)
{
	/* Start the SimCoupe debugger - just console mode for now */
	/* So exit the ui and go back to the normal emulation mode */
	
	disass = !disass;
	ui_main_end();	

}

void
ui_CBselect_drive1(char *TheDiskImageName)
{

	if (strlen(TheDiskImageName))
		insert_disk(TheDiskImageName, 1);

}

void
ui_select_drive1(void)
{

	FileSelectorOpen(TheCwd, "*.dsk", ui_CBselect_drive1, DialogGetActiveDialogObject());

}

void
ui_CBselect_drive2(char *TheDiskImageName)
{

	if (strlen(TheDiskImageName))
		insert_disk(TheDiskImageName, 2);

}


void
ui_select_drive2(void)
{

	FileSelectorOpen(TheCwd, "*.dsk", ui_CBselect_drive2, DialogGetActiveDialogObject());

}

void
ui_about_return(void)
{

	DialogSetActiveDialogToRoot();
	
}

/* Data associated with the Create disk image dialogs */
static char ImageName[BUFSIZ] = "Image.dsk";
static int 	ImageAddSystem;		 


void
ui_main(void)
{
	/* The main user interface to SimCoupe */

	ui_enter();
	
	ClearDisplay();

	/* First Create the top level menus */
	
	TopMenuForm  = FormCreate(0, 0, 255, GLYPH_HEIGHT+8*SH_WID, NULL);
	FileMenu     = MenuCreate(4,   GLYPH_HEIGHT+6*SH_WID, TopMenuForm);	
	SamToolsMenu = MenuCreate(40,  GLYPH_HEIGHT+6*SH_WID, TopMenuForm);	
	OptionsMenu  = MenuCreate(100, GLYPH_HEIGHT+6*SH_WID, TopMenuForm);	
	HelpMenu     = MenuCreate(154, GLYPH_HEIGHT+6*SH_WID, TopMenuForm);	

	/* Create the About dialog (Help -> About)*/
	AboutDialog  = ui_make_about(ui_about_return, HelpMenu);
	
	/* Create the Create Disk image dialog (File -> Create Image) */
	CreateImageDialog = FormCreate(20, 50, 215, 70, FileMenu);
	FormAddTextItem(CreateImageDialog, 6, 12, "File Name :");
	FormAddInputBoxItem(CreateImageDialog, 80, 8, BUFSIZ, 20, ImageName, NULL);
	FormAddTextItem(CreateImageDialog, 60, 30, "Make System Disk");
	FormAddSwitchItem(CreateImageDialog, 44, 32, 2, 2, &ImageAddSystem, True, False);
	FormAddButtonItem(CreateImageDialog, 20, 48, "  OK  ", FORM_DIALOG_BUTTON, NULL);
	FormAddButtonItem(CreateImageDialog, 150, 48,"Cancel", FORM_DIALOG_BUTTON, NULL);

	/* Create the drive selection sub-menu (File -> Change Disk) */
	SelectDriveMenu = MenuCreate(66, 2*GLYPH_HEIGHT+6*SH_WID, FileMenu);	
	MenuAddItem(SelectDriveMenu, MENU_FUNCTION_ITEM, "Disk Drive 1", ui_select_drive1);
	MenuAddItem(SelectDriveMenu, MENU_FUNCTION_ITEM, "Disk Drive 2", ui_select_drive2);
	
	/* Set up the main menu */
	FormAddButtonItem(TopMenuForm, 4,   4, "File", FORM_DIALOG_BUTTON, FileMenu);
	FormAddButtonItem(TopMenuForm, 40,  4, "SamTools", FORM_DIALOG_BUTTON, SamToolsMenu);
	FormAddButtonItem(TopMenuForm, 100, 4, "Options", FORM_DIALOG_BUTTON, OptionsMenu);
	FormAddButtonItem(TopMenuForm, 154, 4, "Help", FORM_DIALOG_BUTTON, HelpMenu);

	MenuAddItem(FileMenu, MENU_SUBMENU_ITEM, "Change Disk", SelectDriveMenu);
#if 0
	MenuAddItem(FileMenu, MENU_DIALOG_ITEM,  "Create Image ", CreateImageDialog);
#endif	
/* No Debug option for vga display - yet */
#ifndef VGA_DISPLAY
	MenuAddItem(FileMenu, MENU_FUNCTION_ITEM, "Start Debug", ui_start_debug);
#endif
	MenuAddItem(FileMenu, MENU_FUNCTION_ITEM, "Exit", ui_main_end);
	
	MenuAddItem(SamToolsMenu, MENU_DIALOG_ITEM, "SamDir", NULL);

	MenuAddItem(OptionsMenu, MENU_DIALOG_ITEM, "The Options", NULL);

	MenuAddItem(HelpMenu, MENU_DIALOG_ITEM, "About", AboutDialog);

	DialogSetActiveDialogObject(TopMenuForm);

}

void
ui_main_end(void)
{
	DialogSetActiveDialogObject(NULL);
	FormFree(TopMenuForm);
	MenuFree(FileMenu);
	MenuFree(SamToolsMenu);
	MenuFree(OptionsMenu);
	MenuFree(HelpMenu);

	FormFree(CreateImageDialog);
	
	ClearDisplay();
	ui_leave();
}

