/* SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File processor.c : z80 main loop and Sam interrupt routines
 *
 * Intel 80x86 assembly optimisations Copyright (c) 1996 Allan Skillman
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/time.h>

#include "z80.h"
#include "mem.h"
#include "resource.h"
#include "util.h"
#include "debug.h"
#include "sam.h"
#include "mouse.h"
#include "interface.h"

#ifndef __GNUC__
#define inline
#endif

#define parity(a) (partable[a])

uns8 partable[256]={
      4, 0, 0, 4, 0, 4, 4, 0, 0, 4, 4, 0, 4, 0, 0, 4,
      0, 4, 4, 0, 4, 0, 0, 4, 4, 0, 0, 4, 0, 4, 4, 0,
      0, 4, 4, 0, 4, 0, 0, 4, 4, 0, 0, 4, 0, 4, 4, 0,
      4, 0, 0, 4, 0, 4, 4, 0, 0, 4, 4, 0, 4, 0, 0, 4,
      0, 4, 4, 0, 4, 0, 0, 4, 4, 0, 0, 4, 0, 4, 4, 0,
      4, 0, 0, 4, 0, 4, 4, 0, 0, 4, 4, 0, 4, 0, 0, 4,
      4, 0, 0, 4, 0, 4, 4, 0, 0, 4, 4, 0, 4, 0, 0, 4,
      0, 4, 4, 0, 4, 0, 0, 4, 4, 0, 0, 4, 0, 4, 4, 0,
      0, 4, 4, 0, 4, 0, 0, 4, 4, 0, 0, 4, 0, 4, 4, 0,
      4, 0, 0, 4, 0, 4, 4, 0, 0, 4, 4, 0, 4, 0, 0, 4,
      4, 0, 0, 4, 0, 4, 4, 0, 0, 4, 4, 0, 4, 0, 0, 4,
      0, 4, 4, 0, 4, 0, 0, 4, 4, 0, 0, 4, 0, 4, 4, 0,
      4, 0, 0, 4, 0, 4, 4, 0, 0, 4, 4, 0, 4, 0, 0, 4,
      0, 4, 4, 0, 4, 0, 0, 4, 4, 0, 0, 4, 0, 4, 4, 0,
      0, 4, 4, 0, 4, 0, 0, 4, 4, 0, 0, 4, 0, 4, 4, 0,
      4, 0, 0, 4, 0, 4, 4, 0, 0, 4, 4, 0, 4, 0, 0, 4
   };


int disass,disass1;

long n_frames;			/* no. of interrupts since powerup */

int LineCycleCounter;		/* Counter used to count the cycles/line */

static int LineInterrupt = True;
static int LineFreq;		/* No of tstates / line generated */
int LineNo; 			/* Scan line being generated */
int InterruptType;		/* Type of z80 interupt to deal with */

static int IntActiveTime;	/* Time (in tstates) an int is active (Port 249 <> 0xff) */
static int MouseStrobeTime;	/* Time (in tstates) from a mouse strobe to a counter reset */

static int DelayedEI = 0;	/* Flag and counter to carry out a delayed */

int flash_phase = 0;		/* gets toggled between 1 and 0 twice per second */

/* Syncronizing timer definitions 				   */
/* NB SyncTimer MUST be declared volatile to prevent the optimizer */
/* Changing the Sync wait loop into an infinite loop               */
static volatile int SyncTimer;
;
static void emulator_sync_timer(int sigparam);
static void emulator_cancel_sync_timer(void);

static struct itimerval TheSyncTimerVal;

/* sigaction structure for the POSIX signal handler */
static struct sigaction TheSignalAction;

/*
 * The main z80 emulation loop
 */
void
emulator_main_loop(void)
{

	register uns8 a, b, c, f, d, e, h, l;

	uns8 r, a1, f1, b1, c1, d1, e1, h1, l1, i, iff1, iff2, im;
	register uns16 pc;
	uns16 ix, iy, sp;
	register unsigned long tstates;
	register unsigned int radjust;
	register uns8 ixoriy, new_ixoriy;
	register uns8 op;

	static uns16 SavePC;
	static uns8  SaveIff1, SaveIff2;

	int refresh_interval;

	register int IntActiveCounter;		/* Counter used for int Active timing */
	register int MouseActiveCounter;	/* Counter used for mouse Active timing */

	/* from (vga/x)samgrx.c */

	extern void (*line_update)(int);
	extern void (*borderline_update)(void);

#ifdef USE_BORDER_EFFECTS
	extern void border_change(void);
        extern int  BorderChangeInInstruction;
#endif
	extern int  BorderTop;
	extern int  BorderBottom;
	extern int  ClutChangeInFrame;
	extern int  BorderChangeInFrame;
	extern int  VmprChangeInFrame;	
        
	/* from samio.c */
	extern void check_events(void);

	/* from mouse.c */
	extern int MouseStrobed;

	n_frames = 0;

	a=f=b=c=d=e=h=l=a1=f1=b1=c1=d1=e1=h1=l1=i=r=iff1=iff2=im=0;
	ixoriy=new_ixoriy=0;
	ix=iy=sp=pc=0;
	tstates=radjust=0;

/*	InterruptType = 0;*/
	
	disass = 0;

	refresh_interval = GetCfg(rr);

	SyncTimer = !GetCfg(sync);

	/* Initialise the sync timer if required */
	if (GetCfg(sync)) {

		/* Add the sync closedown routine to the Quit sequence */
		OnQuit(emulator_cancel_sync_timer);

		/* Set the sync timer to 20000 usecs (1/50 sec frame rate) */
		TheSyncTimerVal.it_interval.tv_sec = 0;
		TheSyncTimerVal.it_interval.tv_usec = 20000;
		TheSyncTimerVal.it_value = TheSyncTimerVal.it_interval;

		/* start sync timer signal */
		TheSignalAction.sa_handler = emulator_sync_timer;
		sigemptyset(&TheSignalAction.sa_mask);
		TheSignalAction.sa_flags = SA_RESTART;

		/* NOTE we can't just use signal() here to get the ALARM   */
		/* signal handler because Linux resets the signal behavior */
		/* when it has been raised, unlike BSD ie using signal in  */
		/* Linux results in the process stopping with 'Alarm clock'*/
		/* the second time the Alarm signal is raised              */
		sigaction(SIGALRM, &TheSignalAction, NULL);
		setitimer(ITIMER_REAL, &TheSyncTimerVal, NULL);

	}

	/* Define the number of tstates/line generated */

	LineFreq = TSTATES_PER_LINE;

	/* Calculate the number of tstates in the 100usecs
	   an interrupt remains active */
	   
	IntActiveTime   = ( 40 * MTSTATES_PER_SECOND);
	MouseStrobeTime = (100 * MTSTATES_PER_SECOND);
	simcoupe_mesg(SIMCOUPE_INFO,"LineFreq        : %d tstates", LineFreq);
    simcoupe_mesg(SIMCOUPE_INFO,"IntActiveTime   : %d tstates", IntActiveTime);
    simcoupe_mesg(SIMCOUPE_INFO,"MouseStrobeTime : %d tstates", MouseStrobeTime);
	LineNo   = 0;	
	IntActiveCounter = 0;

	/* Similar for mouse reset */	
	MouseActiveCounter = 0;

	LineCycleCounter = 0;

#ifdef USE_BORDER_EFFECTS
	BorderChangeInInstruction = False;
#endif
	while(1) {
		if (disass) {
/*			fprintf(stderr, "LineCycleCounter = %d, %d, %d\n",LineCycleCounter,LineNo,hpen);*/
			disass = run_debug(pc);
			if (disass1 == 2) {
				fprintf(stderr, "Z80 CPU State\n-------------\n");
				fprintf(stderr, "  AF  = %04X\t\t  AF' = %04X\t\t  A = %02X  F = %02X\n",
					af, af_prime, a, f);
				fprintf(stderr, "  BC  = %04X\t\t  BC' = %04X\t\t  B = %02X  C = %02X\n",
					bc, bc_prime, b, c);
				fprintf(stderr, "  DE  = %04X\t\t  DE' = %04X\t\t  D = %02X  E = %02X\n",
					de, de_prime, d, e);
				fprintf(stderr, "  HL  = %04X\t\t  HL' = %04X\t\t  H = %02X  L = %02X\n",
					hl, hl_prime, h, l);
				fprintf(stderr, "  IX  = %04X\t\t  IY  = %04X\n\n",
					ix, iy);
		
				fprintf(stderr, "  PC  = %04X\t\t  SP  = %04X\n",
					pc, sp);
	
				fprintf(stderr, "   R  = %02X\t\t   I  = %02X\n",
					r, i);
	
				fprintf(stderr, "  interrupts %s, IM%d\n",
					iff1 ? "enabled" : "disabled", im);
	
				fprintf(stderr, "  FLAGS: C=%d N=%d P/V=%d H=%d Z=%d S=%d\n\n",
					f & F_CARRY    ? 1 : 0,
					f & F_NADD     ? 1 : 0,
					f & F_OVERFLOW ? 1 : 0,
					f & F_HCARRY   ? 1 : 0,
					f & F_ZERO     ? 1 : 0,
					f & F_NEG      ? 1 : 0);
			}
		}


		ixoriy=new_ixoriy;
		new_ixoriy=0;

		DelayedEI = 0;
		op=read_byte(pc);
		pc++;
		radjust++;
		switch(op){

#include "z80ops.c"

		}

		LineCycleCounter += tstates;
		
		/* If we have strobed the mouse, we will need to do
		   a reset after 100 usecs */
		   
		if ((MouseStrobed) && ((MouseActiveCounter+=tstates)>=MouseStrobeTime)) {
			MouseActiveCounter -= MouseStrobeTime;
			MouseStrobed = 0;
			sam_mouse_reset(0);
		}


		if (new_ixoriy) continue;

		/* Call the interrupt if required */

		if (!DelayedEI && (status_reg != 0xff) && iff1) {	
		         if(iff1){
 		           if(op==0x76)pc++;
      			   iff1=iff2=0;
#if 0
			   LineCycleCounter+=8; /* accompanied by an input from the data bus */
#endif
			/* Hack!!! */
				LineCycleCounter+=0;
    			   switch(im){
    			       	case 0: /* IM 0 */
 		              	case 1: /* undocumented */
            			case 2: /* IM 1 */
               			/* there is little to distinguish between these cases */
#if 1
				  LineCycleCounter+=8;
#endif
    				  push2(pc);
		                  pc=0x38;
                		  break;
		               case 3: /* IM 2 */
#if 0
				  LineCycleCounter+=16;
#endif
		                  {
	        	             uns16 addr=read_word((i<<8)|0xff);
		               	     push2(pc);
				     pc=addr;
	                  	  }
		           }
			}
			
		}

		if ((status_reg != 0xff) && ((IntActiveCounter+=tstates)>=IntActiveTime)) {
			IntActiveCounter -= IntActiveTime;
			status_reg = 0xff;
		}
		

		if ((LineNo>58) && LineInterrupt &&
		     (LineCycleCounter >= (LineFreq-BorderWidth_H_1x1))) {

			/* Update the hpen port */
			hpen = LineNo-59;
			LineInterrupt = False;
					
			if ((line_int < 192) && (LineNo == line_int+59)) {
				/* Line Interrupt */

				IntActiveCounter = 0;
				status_reg = 0xfe;

/*				if (disass) {disass = 0;disass1=1;} */
			}
			else if (LineNo == 192+119) {
			
				IntActiveCounter = 0;
				status_reg = 0xf7;
			}

		}


		/* Deal with the display */
		if (LineCycleCounter>=LineFreq) {
			LineCycleCounter -= LineFreq;
			if (LineCycleCounter==4) LineCycleCounter +=4;
			LineInterrupt = True;

			if (LineNo <= 59) {
			/* Top Border */
				hpen = 0;
				attr_reg = 255;

				if (LineNo > BorderTop)
					borderline_update();

			}
			else if (LineNo > 59 && LineNo < 252) {
			/* Main Screen */
			
				line_update(LineNo-60);
				/* Update the lpen port */
/*				hpen = LineNo - 59;			*/
				attr_reg = 0;
			}
			else {
			/* Bottom Border */
				hpen = 192;
				/* Set attr port to 0xff in the border for
				   Spectrum compatability */
				attr_reg = 255;

				if (LineNo < BorderBottom)
					borderline_update();


			}
		
			/* Now we are Generating the next line down so
			   update the counter */
			   
			LineNo++;

			if (LineNo == (192+120)) {
			/* FlyBack - Frame interrupt */

				LineNo = 0;
				
				n_frames++;
				check_events();

				/* Wait for the Sync timer to time out     */
				/* SyncTimer is raised by the timer signal */
				/* in emulator_sync_timer()		   */
				while (!SyncTimer) {
					sigpause(0);
				}
				/* If the sync time is not selected this will set the     */
				/* the timer flag to True, so there will never be a pause */
				SyncTimer = !GetCfg(sync);

				FlyBack();

				/* every 25 frames -> twice/sec. */
				if (((n_frames % 25) == 0) && (VMPR_SCREEN_MODE < MODE_3) &&
				      GetCfg(flashing)) {

				/* Sam modes 1 and 2 use the flashing attribute bit. Every frame the
				   flash_phase is flipped (pen <-> paper in the screen write routines)
				   and ClutChangeInFrame is set to the very last display line to 
				   make sure the whole display is updated
				*/

					flash_phase = 1 - flash_phase;
					ClutChangeInFrame = (192+120);

				}

				/* Deal with NMI type interrupts */
		        	switch(InterruptType) {
		            	 case Z80_none:
		            		break;
		            	 case Z80_nmi:
					if(read_byte(pc)==0x76)pc++;
					iff2=iff1;
       					iff1=0;
        		     	  /* The Z80 performs a machine fetch cycle for 5 Tstates
               	 		  but ignores the result.  It takes a further 10 Tstates
               	   		  to jump to the NMI routine at 0x66. */
               	   		  	LineCycleCounter+=16;
				  	push2(pc);
					pc=0x66;
					InterruptType = Z80_none;
					break;
	            		 case Z80_reset:
					a=f=b=c=d=e=h=l=a1=f1=b1=c1=d1=e1=
					h1=l1=i=r=iff1=iff2=im=0;
					ix=iy=sp=pc=0;
					radjust=0;
					init_pages();
                       			init_ports();
					InterruptType = Z80_none;
					break;
				 /* Special 'interrupt' types which stop and restart */
				 /* the processor for UI routines		     */
				 case Z80_halt:
					SavePC = pc;
					/* Section AB now contains the dialog screen */
					/* Use the memory above this to set up an    */
					/* enless loop (jr -2)			     */
					pc = 0x7ffe;
					write_byte(pc  , 0x18);	/* jr */
					write_byte(pc+1, 0xfe);	/* -2 */
					/* Set the flip-flops so we don't get any    */
					/* (maskable) interupts                      */
					SaveIff1 = iff1;
					SaveIff2 = iff2;
					iff1 = iff2 = 0;
					InterruptType = Z80_none;
				 	break;
				 case Z80_restart:
					pc = SavePC;
					iff1 = SaveIff1;
					iff2 = SaveIff2;
					InterruptType = Z80_none;
					BorderChangeInFrame = (192+120);
				 	break;
		    		 default:
		    			break;
			 	}

			}
			
#ifdef USE_BORDER_EFFECTS
			border_change();
#endif						
			/* Reset the display flags if required */
			if (LineNo == VmprChangeInFrame-1)
				VmprChangeInFrame = False;
			if (LineNo == ClutChangeInFrame-1)
				ClutChangeInFrame = False;
			if (LineNo == BorderChangeInFrame-1)
				BorderChangeInFrame = False;

		
                }
                
#ifdef USE_BORDER_EFFECTS
		if (BorderChangeInInstruction) {
			border_change();
			BorderChangeInInstruction = False;
		}
#endif
	}

}

void
emulator_sync_timer(int sigparam)
{
	/* Reset sync timer flag */
	SyncTimer = True;
}

void
emulator_cancel_sync_timer(void)
{
	/* Cancel the sync timer signal - called a closedown */

	static struct itimerval TmpTimer = {
					    {0L, 0L},
					    {0L, 0L}
					  };
	signal(SIGALRM, SIG_IGN);
	setitimer(ITIMER_REAL, &TmpTimer, NULL);
}
