/* 
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File sam.h - public interface to sam hardware emulation code.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _SAM_H
#define _SAM_H

void init_display(void);
void init_ports(void);
void out_byte(uns16 port, uns8 val);
uns8 in_byte(uns16 port);
void check_events(void);
void ChangeScaleMode(int SF);
int init_display_depth(int DisplayDepth);
int get_colours(void);
int image_init(void); 

#ifdef VGA_DISPLAY
void FlyBack(void);
#else
void (*FlyBack)(void);
#endif
void (*border_change)(void);

/* a list of actions which can be bound to keypresses */
enum {
	KM_JOY_UP, KM_JOY_DOWN,
	KM_JOY_LEFT, KM_JOY_RIGHT,
	KM_JOY_FIRE,
	NUM_KEYMAPPINGS
};

/* vgalib modes for 1x1 and 2x2 scaling */

#define VGA1X1MODE	G320x200x256
#define VGA2X2MODE	G640x480x256

/* default dimentions - scaled accordingly in vgasam.c */

#ifdef VGA_DISPLAY
#define HSize1x1		320
#define HSize2x2		640
#define VSize1x1		200
#else
#define HSize1x1		304
#define HSize2x2		608
#define VSize1x1		240
#endif
#define VSize2x2		480

#define BorderWidth_H_1x1	((HSize1x1-256)/2)
#define BorderWidth_V_1x1	((VSize1x1-192)/2)
#define BorderWidth_H_2x2	(2*BorderWidth_H_1x1)
#define BorderWidth_V_2x2	(2*BorderWidth_V_1x1)

/* Mouse device - should normally be /dev/mouse (only used for vga display) */

#define MOUSE_DEVICE	"/dev/mouse"

#ifdef PSEUDO_IO
#define PSEUDO_IO_LOWBYTE 0xbf
#define PSEUDO_STDIN  (0x0000 | PSEUDO_IO_LOWBYTE)
#define PSEUDO_STDOUT (0x0100 | PSEUDO_IO_LOWBYTE)
#define PSEUDO_STDERR (0x0200 | PSEUDO_IO_LOWBYTE)
#endif

/* The Number of Colours the SAM has (128 of course) */
#define NCOLOURS 128

/* Macros for access to sam ports */

#define SOUND_ADDR	511
#define SOUND_DATA	252

#define VMPR_PORT	252
#define HMPR_PORT	251
#define LMPR_PORT	250
#define MIDI_PORT	253
#define BORDER_PORT	254
#define LINE_PORT	249
#define CLUT_BASE_PORT	248
#define ATTR_PORT	255
#define KEYBOARD_PORT	254
#define STATUS_PORT	249
#define LPEN_PORT	248
#define HPEN_PORT	504
#define CLOCK_PORT	239

#define LEPR_PORT	128
#define HEPR_PORT	129

#define DISK1_PORT	224
#define DISK2_PORT	240

#define VMPR_SCREEN_PAGE	(vmpr & 0x1f)
#define VMPR_SCREEN_MODE        (vmpr & 0x60)
#define HMPR_PAGE		(hmpr & 0x1f)
#define HMPR_MD3COL	       ((hmpr & 0x60) >> 3)
#define HMPR_MCNTRL		(hmpr & 0x80)
#define LMPR_PAGE		(lmpr & 0x1f)
#define LMPR_RAM0		(lmpr & 0x20)
#define LMPR_ROM1		(lmpr & 0x40)
#define LMPR_WPRAM		(lmpr & 0x80)

#define BORD_VAL(x)	       ((((x) & 0x20 ) >> 2) | ((x) & 0x07))
#define BORD_SOFF		(border & 0x80)

#define MD3_SWITCH(x)	       (((x) == 0x01 || (x) == 0x02) ? (x) ^ 0x03 : (x))

#define MODE_1			0x00
#define MODE_2			0x20
#define MODE_3			0x40
#define MODE_4			0x60

/* The sam ports */

/* Read / Write ports */

extern uns8 vmpr;
extern uns8 hmpr;
extern uns8 lmpr;
extern uns8 midi;
extern uns8 border;

extern uns8 lepr;
extern uns8 hepr;

extern uns8 clock_reg;

/* Write only ports */

extern uns8 line_int;
extern uns32 clut[];

/* Read only ports */

extern uns8 attr_reg;
extern uns8 keyboard;
extern uns8 status_reg;
extern uns8 lpen;
extern uns8 hpen;

#endif
